package cz.frantovo.gui.tabulky;

import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.ImageIcon;
import javax.swing.table.TableModel;

/**
 * <p>Tato třída umožňuje jednoduchým způsobem přidat vašim tabulkám funkci
 * řazení. Stačí ve vaší třídě nahradit: <br>
 * "import javax.swing.JTable;"<br>
 * tímto importem: "import cz.frantovo.gui.tabulky.JTable;"<br>
 * </p>
 * <p>Pomocí klávesy ctrl jde řadit podle více sloupců</p>
 * <p>Také přidává funkci zobrazování tooltipu (zobrazuje obsah buňky)</p>
 * 
 * <p>Protože třídící model a skutečný model obsahují řádky v jiném pořadí,
 * je potřeba přepočítat index získaný metodou getSelectedRow() na index ve skutečném modelu</p>
 * <p>
 * k tomu sloužá tento postup:</p>
 * <pre>
 * int r = jTable1.getSelectedRow();
 * int rr = ((TableSorterModel)tabulka.getModel()).modelIndex(r);
 * </pre>
 * <p>rr je potom index řádku ve skutečném modelu.</p>
 * 
 * @author František Kučera
 */
public class JTable extends javax.swing.JTable {

    private static final long serialVersionUID = -5133441062459764995L;
    private TableSorterModel tableSorterModel;
    private TableModel realTableModel;
    private boolean showTooltips = false;

    public boolean isShowTooltips() {
        return showTooltips;
    }

    /** Nastavuje, zda se mají zobrazovat tooltipy zobrazující obsah buňky */
    public void setShowTooltips(boolean showTooltips) {
        this.showTooltips = showTooltips;
    }

    /**
     * Nastaví TableModel a zabalí ho do TableSorterModelu, tabulka tím
     * získá schopnost řazení řádků
     */
    @Override
    public void setModel(TableModel dataModel) {
        realTableModel = dataModel;
        tableSorterModel = new TableSorterModel(dataModel);

        tableSorterModel.setTableHeader(getTableHeader());

        super.setModel(tableSorterModel);
    }

    /**
     * @return TableModel, která obsahuje data. Není tedy zabalen do
     *         TableSorterModelu
     */
    public TableModel getRealTableModel() {
        return realTableModel;
    }

    /**
     * @return TableSorterModel, který obaluje skutečný TableModel, který
     *         obsahuje data. TableSorterModel slouží pouze k řazení položek
     */
    @Override
    public TableModel getModel() {
        return super.getModel();
    }

    /**
     * Pokud je nastaveno showTooltips, zobrazuje v tooltipu obsah buňky pod
     * myší (hodí se, pokud je obsah buňky příliš dlouhý). Pokud buňka
     * obsahuje ImageIcon, pak tooltip zobrazuje ImageIcon.getDescription()
     */
    @Override
    public String getToolTipText(MouseEvent event) {
        if (showTooltips) {
            Point p = event.getPoint();
            int col = columnAtPoint(p);
            int rw = rowAtPoint(p);
            if (col >= 0 && rw >= 0) {
                Object o = getValueAt(rw,
                        col);
                if (o != null) {
                    if (o instanceof ImageIcon) {
                        return ((ImageIcon) o).getDescription();
                    } else {
                        return o.toString();
                    }
                }
            }
            return super.getToolTipText();
        } else {
            return super.getToolTipText();
        }
    }
}
